/*
 *     SimpleTest.java  Sep 6, 2001
 *
 *     Katsuhisa Maruyama (maru@fse.cs.ritsumei.ac.jp)
 */

import jp.ac.ritsumei.cs.fse.jrt.model.*;
import jp.ac.ritsumei.cs.fse.jrt.parser.JavaModelFactory;
import jp.ac.ritsumei.cs.fse.jrt.parser.summary.Summary;
import jp.ac.ritsumei.cs.fse.jrt.parser.SimpleNode;
import jp.ac.ritsumei.cs.fse.jrt.parser.SimpleVisitor;
import jp.ac.ritsumei.cs.fse.jrt.parser.JavaParserVisitor;
import jp.ac.ritsumei.cs.fse.jrt.parser.ast.*;
import java.util.*;
import java.io.*;

public class SimpleTest extends SimpleVisitor {
    private int curDepth = 0;
    private static JavaFile jfile;

    public static void main(String args[]) {
        if (args.length == 1) {
            String fileName = args[0];

            String dirName = ".";
            if (fileName.indexOf(File.separator) != -1) {
                dirName = fileName.substring(0, fileName.lastIndexOf(File.separator));
            }

            String JDKFilePath;
            if (System.getProperty("os.name").startsWith("Windows")) {
                JDKFilePath = "c:\\jdk1.3.1\\src.jar";
            } else {
                JDKFilePath = "/usr/local/j2sdk1_3_1/src.jar";
            }
            JavaModelFactory.getInstance().setJDKFile(JDKFilePath);
            Summary.getInstance().parse(dirName);

            // JavaModelFactory.getInstance().addWarningEventListener(new ExceptionPrint());

            jfile = JavaModelFactory.getInstance().getEachJavaFile(fileName);

            if (jfile != null) {
                JavaParserVisitor visitor = new SimpleTest();
                jfile.accept(visitor);
            }
        } else {
            System.err.println("Usage: java jp.ac.ritsumei.cs.fse.jrt.model.SimpleTest inputfile");
            return;
        }
    }

    public Object visit(ASTCompilationUnit node, Object data) {
        JavaFile jfile = (JavaFile)data;
        print("----");
        print("File: " + jfile.getName());
        Object obj = node.childrenAccept(this, jfile);
        return obj;
    }

    public Object visit(ASTPackageDeclaration node, Object data) {
        JavaFile jfile = (JavaFile)data;
        print("Package: " + jfile.getPackageName());
        Object obj = node.childrenAccept(this, jfile);
        return obj;
    }

    public Object visit(ASTUnmodifiedClassDeclaration node, Object data) {
        JavaClass jclass = node.getJavaClass();
        print("Class: " + jclass.getModifier().toString() + " " + jclass.getQualifiedName());
        print("  extends " + jclass.getSuperClassName());
        print("  implements " + jclass.getSuperClassNameList());
        print("{");
        curDepth++;
        Object obj = node.childrenAccept(this, data);
        curDepth--;
        print("} class end");
        return obj;
    }

    public Object visit(ASTUnmodifiedInterfaceDeclaration node, Object data) {
        JavaClass jclass = node.getJavaClass();
        print("Interface: " + jclass.getModifier().toString() + " " + jclass.getName());
        print("  extends " + jclass.getSuperClassNameList());
        print("{");
        curDepth++;
        Object obj = node.childrenAccept(this, data);
        curDepth--;
        print("} interface end");
        return obj;
    }

    public Object visit(ASTVariableDeclarator node, Object data) {
        JavaStatement jst = node.getJavaStatement();
        JavaVariable jvar = jst.getDefVariables().getFirst();
        Object obj = node.childrenAccept(this, jvar);
        print("Variable Declaration: " + jvar.getModifier().toString() + " "
              + jvar.getNameWithID() + "#" + jvar.getType()
              + " = " + jst.getUseVariables().toStringWithType());
        return obj;
    }

    public Object visit(ASTMethodDeclaration node, Object data) {
        Object obj = node.childrenAccept(this, data);
        curDepth--;
        print("} method end");
        return obj;
    }

    public Object visit(ASTMethodDeclarator node, Object data) {
        JavaMethod jmethod = node.getJavaMethod();
        print("Method: " + jmethod.getModifier().toString() + " "
              + jmethod.getType() + " " + jmethod.getName());
        print("Method Signature: " + jmethod.getSignature());

        Iterator it = jmethod.getParameters().iterator();
        while (it.hasNext()) {
            JavaStatement jst = (JavaStatement)it.next();
            JavaVariable jvar = jst.getDefVariables().getFirst();
            print("Formal Parameter: " + jvar.getModifier().toString()
               + " " + jvar.getType() + " " + jvar.getNameWithID());
        }

        print("{");
        curDepth++;
        Object obj = node.childrenAccept(this, data);
        return obj;
    }

    public Object visit(ASTConstructorDeclaration node, Object data) {
        JavaMethod jmethod = node.getJavaMethod();
        print("Constructor: " + jmethod.getModifier().toString() + " "
              + " " + jmethod.getName());
        print("Method Signature: " + jmethod.getSignature());

        Iterator it = jmethod.getParameters().iterator();
        while (it.hasNext()) {
            JavaStatement jst = (JavaStatement)it.next();
            JavaVariable jvar = jst.getDefVariables().getFirst();
            print("Formal Parameter: " + jvar.getModifier().toString()
               + " " + jvar.getType() + " " + jvar.getNameWithID());
        }

        print("{");
        curDepth++;
        Object obj = node.childrenAccept(this, data);    
        curDepth--;
        print("} constructor end");
        return obj;
    }

    public Object visit(ASTExpression node, Object data) {
        JavaStatement jst = node.getJavaStatement();
        if (!jst.getDefVariables().isEmpty()) {
            print("Expression: " + " " + jst.getDefVariables().toString() + " = "
                  + jst.getUseVariables().toString());
        }

        Object obj = node.childrenAccept(this, data);
        return obj;
    }

    public Object visit(ASTPreIncrementExpression node, Object data) {
        JavaStatement jst = node.getJavaStatement();
        if (!jst.getDefVariables().isEmpty()) {
            print("Expression: " + " " + jst.getDefVariables().toString() + " = "
                  + jst.getUseVariables().toString());
        }

        Object obj = node.childrenAccept(this, data);
        return obj;
    }

    public Object visit(ASTPreDecrementExpression node, Object data) {
        JavaStatement jst = node.getJavaStatement();
        if (!jst.getDefVariables().isEmpty()) {
            print("Expression: " + " " + jst.getDefVariables().toString() + " = "
                  + jst.getUseVariables().toString());
        }

        Object obj = node.childrenAccept(this, data);
        return obj;
    }

    public Object visit(ASTPostfixExpression node, Object data) {
        JavaStatement jst = node.getJavaStatement();
        if (!jst.getDefVariables().isEmpty()) {
            print("Expression: " + " " + jst.getDefVariables().toString() + " = "
                  + jst.getUseVariables().toString());
        }

        Object obj = node.childrenAccept(this, data);
        return obj;
    }

    public Object visit(ASTArguments node, Object data) {
        Object obj = node.childrenAccept(this, data);
        JavaStatement jst = node.getJavaStatement();

        print("Call: " + " " + jst.getDefVariables().toStringWithType() + " = "
              + jst.getUseVariables().toStringWithType());

        Iterator it = node.getArguments().iterator();
        while (it.hasNext()) {
            jst = (JavaStatement)it.next();
            print("Arguments: " + jst.getUseVariables().toString());
        }
        return obj;
    }

    public Object visit(ASTLabeledStatement node, Object data) {
        print("Label statement: " + node.getName());
        Object obj = node.childrenAccept(this, data);
        return obj;
    }

    public Object visit(ASTStatementExpression node, Object data) {
	Object obj = node.childrenAccept(this, data);
        JavaStatement jst = node.getJavaStatement();
        if (!jst.getDefVariables().isEmpty()) {
            print("Expression: " + jst.getDefVariables().toStringWithType()
                  + " = " + jst.getUseVariables().toStringWithType());
        }
        return obj;
    }

    public Object visit(ASTSwitchStatement node, Object data) {
        print("Switch {");
        Object obj = node.childAccept(this, data, 0);
        JavaStatement jst = node.getJavaStatement();
	print("Expression: " + jst.getDefVariables().toString()
              + " = " + jst.getUseVariables().toString());

        for (int i = 1; i < node.jjtGetNumChildren(); i++) {
            curDepth++;
            if (node.jjtGetChild(i) instanceof ASTBlockStatement) {
                curDepth++;
                obj = node.childAccept(this, data, i);
                curDepth--;
            } else {
                obj = node.childAccept(this, data, i);
            }
            curDepth--;
        }
        print("} switch end");
        return obj;
    }

    public Object visit(ASTSwitchLabel node, Object data) {
        JavaStatement jst = node.getJavaStatement();
        print("Case Expression: " + jst.getDefVariables().toString()
              + " = " + jst.getUseVariables().toString());
        Object obj = node.childrenAccept(this, data);
        return obj;
    }

    public Object visit(ASTIfStatement node, Object data) {
        int num = node.jjtGetNumChildren();

        print("If {");
        curDepth++;
        Object obj = node.childAccept(this, data, 0);
        JavaStatement jst = node.getJavaStatement();
        print("Expression: " + jst.getDefVariables().toString()
              + " = " + jst.getUseVariables().toString());
        curDepth--;

        print("Then:");
        curDepth++;
        obj = node.childAccept(this, data, 1);
        curDepth--;

        if (num > 2) {
            print("Else:");
            curDepth++;
            obj = node.childAccept(this, data, 2);
            curDepth--;
        }
        print("} if end");

        return obj;
    }

    public Object visit(ASTWhileStatement node, Object data) {
        print("While {");
        curDepth++;
        Object obj = node.childAccept(this, data, 0);
        JavaStatement jst = node.getJavaStatement();
        print("Expression: " + jst.getDefVariables().toString()
              + " = " + jst.getUseVariables().toString());
        curDepth--;

        print("Do:");        
        curDepth++;
        obj = node.childAccept(this, data, 1);
        curDepth--;
        print("} while end");

        return obj;
    }

    public Object visit(ASTDoStatement node, Object data) {
        print("Do {");
        curDepth++;
        Object obj = node.childAccept(this, data, 0);
        curDepth--;

        print("While:");
        curDepth++;
        obj = node.childAccept(this, data, 1);
        JavaStatement jst = node.getJavaStatement();
        print("Expression: " + jst.getDefVariables().toString()
              + " = " + jst.getUseVariables().toString());
        curDepth--;
        print("} do end");

        return obj;
    }

    public Object visit(ASTForStatement node, Object data) {
        Object obj = null;
        print("For {");
        for (int i = 0; i < node.jjtGetNumChildren(); i++) {
            curDepth++;
            obj = node.childAccept(this, data, i);
            if (node.jjtGetChild(i) instanceof ASTExpression) {
                JavaStatement jst = node.getJavaStatement();
                print("Expression: " + jst.getDefVariables().toString()
                      + " = " + jst.getUseVariables().toString());
            }
            curDepth--;
        }
        print("} for end");
        return obj;
    }

    public Object visit(ASTBreakStatement node, Object data) {
        print("Break {");
        curDepth++;
        Object obj = node.childrenAccept(this, data);
        curDepth--;
        print("} break end");
        return obj;
    }

    public Object visit(ASTContinueStatement node, Object data) {
        print("Continue {");
        curDepth++;
        Object obj = node.childrenAccept(this, data);
        curDepth--;
        print("} continue end");
        return obj;
    }

    public Object visit(ASTReturnStatement node, Object data) {
        print("Return {");
        curDepth++;
        Object obj = node.childrenAccept(this, data);
        JavaStatement jst = node.getJavaStatement();
        print(jst.getDefVariables().toStringWithPosition()
              + " = " + jst.getUseVariables().toString());
        curDepth--;
        print("} return end");
        return obj;
    }

    // The followings are not treated.
    // public Object visit(ASTThrowStatement node, Object data) {}
    // public Object visit(ASTSynchronizedStatement node, Object data) {}
    // public Object visit(ASTTryStatement node, Object data) {}

    private void print(String message) {
        for (int i = 0; i < curDepth; i++) {
            System.out.print("    ");
        }
        System.out.println(message);
    }
}
