/*
 *     PDGTest.java  Nov 22, 2001
 *
 *     Katsuhisa Maruyama (maru@fse.cs.ritsumei.ac.jp)
 */

import jp.ac.ritsumei.cs.fse.jrt.parser.*;
import jp.ac.ritsumei.cs.fse.jrt.graphs.GraphFactory;
import jp.ac.ritsumei.cs.fse.jrt.graphs.util.*;
import jp.ac.ritsumei.cs.fse.jrt.graphs.cfg.*;
import jp.ac.ritsumei.cs.fse.jrt.graphs.pdg.*;
import jp.ac.ritsumei.cs.fse.jrt.parser.JavaModelFactory;
import jp.ac.ritsumei.cs.fse.jrt.parser.summary.Summary;
import jp.ac.ritsumei.cs.fse.jrt.model.*;
import jp.ac.ritsumei.cs.fse.jrt.test.FileCompare;
import java.util.*;
import java.io.*;
import junit.framework.*;

public class PDGTest extends TestCase implements GraphEdgeSort {
    private FileWriter fileWriter;

    public PDGTest(String name) {
        super(name);
    }

    public static void main(String args[]) {
        junit.textui.TestRunner.run(suite());
    }

    protected void setUp() {
        String JDKFilePath;
        if (System.getProperty("os.name").startsWith("Windows")) {
            JDKFilePath = "c:\\jdk1.3.1\\src.jar";
        } else {
            JDKFilePath = "/usr/local/j2sdk1_3_1/src.jar";
        }
        JavaModelFactory.getInstance().setJDKFile(JDKFilePath);
    }

    protected void tearDown() {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite();

        List testcases = new ArrayList();
        testcases.add(new String("./testcases"));

        List jfiles = makeTestCases(testcases);
        Iterator it = jfiles.iterator();
        while (it.hasNext()) {
            final JavaFile jfile = (JavaFile)it.next();
            suite.addTest(new PDGTest("PDG Test" + jfile.getName()) {
                protected void runTest() {
                    testOutputFile(jfile);
                }
            });
        }
        return suite;
    }

    private static List makeTestCases(List testcases) {
        List jfiles = new ArrayList();
        Iterator it = testcases.iterator();
        while (it.hasNext()) {
            String dir = (String)it.next();
            Summary.getInstance().parse(dir);

            File directory = new File(dir);
            if (directory.isDirectory()) {
                String[] names = directory.list();
                for (int i = 0; i < names.length; i++) {
                    String name = dir + File.separator + names[i];

                    File file = new File(name);
                    if (file.isFile() && name.endsWith(".java")) {
                        JavaFile jfile = JavaModelFactory.getInstance().getEachJavaFile(name);
                        jfiles.add(jfile);
                    }
                }
            }
        }
        return jfiles;
    }

    public void testOutputFile(JavaFile jfile) {
        GraphNode.clearID();
        GraphFactory.getInstance().makeCFG(jfile);
        GraphFactory.getInstance().makePDG(jfile);
        File file = new File(jfile.getShortFileName() + ".pdg");
        // print(jfile);
        write(jfile, file);
        assertTrue(FileCompare.compare(jfile.getName() + ".pdg", file.getName()));
        file.delete();
    }

    private void print(JavaFile jfile) {
        Iterator it = jfile.getJavaClasses().iterator();
        while (it.hasNext()) {
            JavaClass jclass = (JavaClass)it.next();
            printClDG(jclass);
        }
    }

    private void printClDG(JavaClass jclass) {
        Iterator it = jclass.getJavaMethods().iterator();
        while (it.hasNext()) {
            JavaMethod jmethod = (JavaMethod)it.next();
            PDG pdg = (PDG)jmethod.getPDG();
            pdg.printCDG();
        }
    }

    private void write(JavaFile jfile, File file) {
        try {
            fileWriter = new FileWriter(file.getName());
            writePDG(jfile);
            fileWriter.close();
        } catch (IOException e) { }
    }

    private void writePDG(JavaFile jfile) {
        Iterator it = jfile.getJavaClasses().iterator();
        while (it.hasNext()) {
            JavaClass jclass = (JavaClass)it.next();
            writePDG(jclass);
        }
    }

    private void writePDG(JavaClass jclass) {
        Iterator it = jclass.getJavaMethods().iterator();
        while (it.hasNext()) {
            JavaMethod jmethod = (JavaMethod)it.next();
            writeGraph(jmethod.getPDG());
        }
    }

    public void writeGraph(Graph graph) {
        writeEdges(graph);
    }

    public void writeEdges(Graph graph) {
        Iterator it = graph.getEdges().iterator();
        while (it.hasNext()) {
            Dependence edge = (Dependence)it.next();

            String info = "unknown";
            switch (edge.getSort()) {
                case trueControlDependence: info = "T"; break;
                case falseControlDependence: info = "F"; break;
                case fallControlDependence: info = "Fall"; break;

                case defuseDependence: info = "DU"; break;
                case deforderDependence: info = "DO"; break;
                case outputDependence: info = "OD"; break;
                case antiDependence: info = "OD"; break;
            }

            if (edge.isCD()) {
                CD e = (CD)edge;
                write(e.getSrcNode().getID() +  "-> " + e.getDstNode().getID()
                      + ": " + info);
            }
            if (edge.isDD()) {
                DD e = (DD)edge;
                write(e.getSrcNode().getID() + " -> " + e.getDstNode().getID()
                      + ": " + info + ": " + e.getVariable().getName());
                if (e.isLoopCarried()) {
                    write("LC:" + e.getLoopCarriedNode().getID());
                }
            }
        }
    }

    private void write(String str) {
        try {
            fileWriter.write(str + "\n");
        } catch (IOException e) { }
    }
}
